@@ -1,6 +1,6 @@
 # /usr/bin/r
 #
-# Copyright 2018-2019 Steven E. Pav. All Rights Reserved.
+# Copyright 2018-2024 Steven E. Pav. All Rights Reserved.
 # Author: Steven E. Pav 
 #
 # This file is part of ohenery.
@@ -19,7 +19,7 @@
 # along with ohenery.  If not, see <http://www.gnu.org/licenses/>.
 #
 # Created: 2018.12.20
-# Copyright: Steven E. Pav, 2018-2019
+# Copyright: Steven E. Pav, 2018-2024
 # Author: Steven E. Pav <shabbychef@gmail.com>
 # Comments: Steven E. Pav
 
@@ -47,14 +47,19 @@ setOldClass('hensm')
 #2FIX: why isn't there a experts version of this one?
 #  @param ngamma  the number of gammas to model; we model
 #        \eqn{\gamma_2} through \eqn{\gamma_n}.
-.hmfit <- function(y, g, X, wt=NULL, eta0=NULL, normalize_wt=FALSE,
+.hmfit <- function(y, g, X, wt=NULL, eta0=NULL, beta0=NULL, gamma0=NULL, normalize_wt=FALSE,
 									 ngamma=4,  method=c('BFGS','NR','CG','NM')) {
 	method <- match.arg(method)
+	if (!is.null(gamma0)) {
+		ngamma <- length(gamma0) + 1
+	} else {
+		gamma0 <- array(1,ngamma-1)
+	}
 	stopifnot(ngamma >= 2)
-#2FIX: allow beta0 input.
 	k <- ncol(X)
-	beta0 <- array(0,k)
-	gamma0 <- array(1,ngamma-1)
+	if (is.null(beta0)) {
+		beta0 <- array(0,k)
+	}
 	theta0 <- c(beta0,gamma0)
 
 	if (!is.null(wt) && normalize_wt) { wt <- wt / abs(mean(wt,na.rm=TRUE)) }  # by having the abs, negative weights still throw an error.
@@ -108,6 +113,17 @@ setOldClass('hensm')
 #' @param na.action  How to deal with missing values in \code{y}, \code{g},
 #' \code{X}, \code{wt}, \code{eta0}.
 #' @param ngamma  The number of gammas to fit. Should be at least 2.
+#' @param fit0    An optional object of class \code{hensm} or of \code{harsm} 
+#' with the initial fit estimates. 
+#' These will be used for \sQuote{warm start} of the estimation procedure. 
+#' A warm start should only speed up estimation, not change the ultimate results. 
+#' When there is mismatch between the coefficients in \code{fit0} and the model 
+#' being fit here, the missing coefficients are initialized as zero. 
+#' If \code{ngamma} is \code{NULL} and \code{fit0} is given, 
+#' we default to the number of gammas in the initial fit, otherwise
+#' we fill any missing gammas with 1.
+#' If a \code{harsm} object is given, then \code{ngamma} must be non-null.
+#'
 #' @template etc
 #' @template note-ties
 #' @template note-weights
@@ -155,40 +171,74 @@ setOldClass('hensm')
 #' hensm(Finish ~ eta0,data=df,group=EventId,weights=weights,ngamma=3)
 #' 
 #' # look for age effect not captured by consensus odds.
-#' hensm(Finish ~ offset(eta0) + fac_age,data=df,group=EventId,weights=weights,ngamma=2)
+#' fmla <- Finish ~ offset(eta0) + fac_age
+#' fit0 <- hensm(fmla,data=df,group=EventId,weights=weights,ngamma=2)
+#' # allow warm start.
+#' fit1 <- hensm(fmla,data=df,group=EventId,weights=weights,fit0=fit0,ngamma=2)
+#' # allow warm start with more gammas.
+#' fit2 <- hensm(fmla,data=df,group=EventId,weights=weights,fit0=fit0,ngamma=3)
+#' # or a different formula
+#' fit3 <- hensm(update(fmla,~ . + PostPosition),data=df,group=EventId,weights=weights,fit0=fit0)
 #'
+#' # warm start from harsm object
+#' fit0_har <- harsm(fmla,data=df,group=EventId,weights=weights)
+#' fit4 <- hensm(fmla,data=df,group=EventId,fit0=fit0_har,weights=weights)
 #'
 #' @importFrom stats coef formula model.frame model.matrix na.omit
 #' @export
 #' @rdname hensm
-hensm <- function(formula,data,group=NULL,weights=NULL,ngamma=4,na.action=na.omit) {
+hensm <- function(formula,data,group=NULL,weights=NULL,ngamma=4,fit0=NULL,na.action=na.omit) {
 	substitute(formula)
 
 	# I find it highly offensive that this cannot be done reasonably
 	# easily in a subfunction because of NSE whatever.
 
 	# https://stackoverflow.com/q/53827563/164611
-  mf <- match.call(expand.dots = FALSE)
-  #turn weights into symbol if character is passed
-  if (is.character(mf$weights)) mf$weights <- as.symbol(mf$weights)
-  if (is.character(mf$group)) mf$group <- as.symbol(mf$group)
-  m <- match(c("formula", "data", "weights", "group", "na.action"), names(mf), 0L)
-  mf <- mf[c(1L, m)]
-  mf$drop.unused.levels <- TRUE 
-  mf[[1L]] <- quote(stats::model.frame) 
-  mf <- eval(mf, parent.frame()) #evaluate call
+	mf <- match.call(expand.dots = FALSE)
+	#turn weights into symbol if character is passed
+	if (is.character(mf$weights)) mf$weights <- as.symbol(mf$weights)
+	if (is.character(mf$group)) mf$group <- as.symbol(mf$group)
+	m <- match(c("formula", "data", "weights", "group", "na.action"), names(mf), 0L)
+	mf <- mf[c(1L, m)]
+	mf$drop.unused.levels <- TRUE 
+	mf[[1L]] <- quote(stats::model.frame) 
+	mf <- eval(mf, parent.frame()) #evaluate call
 
 	Xs <- model.matrix(formula,mf)
 	# remove intercept!
 	if (colnames(Xs)[1] == '(Intercept)') { Xs <- Xs[,-1,drop=FALSE] }
 	y <- as.vector(model.response(mf))
-  group <- as.vector(model.extract(mf,'group'))
+	group <- as.vector(model.extract(mf,'group'))
 	eta0 <- model.offset(mf)
-  wt <- as.vector(model.weights(mf))
+	wt <- as.vector(model.weights(mf))
 
 	dat <- list(Xs=Xs,y=y,group=group,eta0=eta0,wt=wt)
 
-	retv <- .hmfit(y=dat$y, g=dat$group, X=dat$Xs, wt=dat$wt, ngamma=ngamma, eta0=dat$eta0)
+	if (!is.null(fit0)) {
+		stopifnot(any(c("hensm","harsm") %in% class(fit0)))
+		feat_names <- colnames(dat$Xs)
+		beta0 <- rep(0,ncol(dat$Xs))
+		found <- feat_names %in% attr(fit0$beta,'names')
+		beta0[found] <- fit0$beta[feat_names[found]]
+		# 2FIX: warn if some names are missing? not needed, I would think.
+		# stopifnot(colnames(dat$Xs) == attr(fit0$beta,'names'))
+		if (is.null(ngamma)) {
+			stopifnot(any(c("hensm") %in% class(fit0)))
+			gamma0 <- fit0$gammas
+			ngamma <- length(gamma0) + 1
+		} else {
+			gamma0 <- rep(1,ngamma-1)
+			if (any(c("hensm") %in% class(fit0))) {
+				dotake <- min(ngamma-1,length(fit0$gammas))
+				gamma0[1:dotake] <- fit0$gammas[1:dotake]
+			}
+		}
+	} else {
+		beta0 <- NULL
+		gamma0 <- NULL
+	}
+
+	retv <- .hmfit(y=dat$y, g=dat$group, X=dat$Xs, wt=dat$wt, beta0=beta0, gamma0=gamma0, ngamma=ngamma, eta0=dat$eta0)
 	retv <- as.linodds(retv, formula, beta=retv$beta)
 	retv
 }
@@ -211,6 +261,7 @@ vcov.hensm <- function(object, ...) {
 #' @export
 #' @importFrom stats printCoefmat
 #' @importFrom methods show
+#' @inheritParams base::print
 #' @rdname hensm
 #' @method print hensm
 print.hensm <- function(x, ...) {
