@@ -508,6 +508,11 @@ test_that("harsmfit prediction",{#FOLDUP
 	expect_error(barfuh <- predict(fitnum,newdata=data,type=ttype,group='race'),NA)
 	expect_equal(fuhbar,barfuh,tolerance=1e-7)
 
+	# if predict data is missing some columns, should error
+	baddata <- data[c('race','V1')]
+	ttype <- 'eta'
+	expect_error(fuhbar <- predict(fitnum,newdata=baddata,type=ttype,group=race))
+
 	# deal with na actions
 	expect_error(fuh <- as.numeric(predict(fitlet,newdata=data,type='eta',group=letrace,na.action=na.pass)),NA)
 	expect_equal(length(fuh),nrow(data))
@@ -531,10 +536,6 @@ test_that("harsmfit prediction",{#FOLDUP
 	expect_error(fuh <- as.numeric(predict(fitlet,newdata=badata,type='eta',group=letrace,na.action=na.omit)),NA)
 	expect_equal(length(fuh),sum(!is.na(badata$V1)))
 	expect_true(all(!is.na(fuh)))
-
-
-
-
 })#UNFOLD
 test_that("hensm bits",{#FOLDUP
 	# travis only?
@@ -604,6 +605,11 @@ test_that("hensm bits",{#FOLDUP
 	expect_error(fitlet <- hensm(outcome ~ V1 + V2,data,group=letrace),NA)
 	expect_error(fitfac <- hensm(outcome ~ V1 + V2,data,group=facrace),NA)
 	expect_error(fitint <- hensm(outcome ~ V1 + V2,data,group=intrace),NA)
+	
+	# if predict data is missing some columns, should error
+	baddata <- data[c('race','V1')]
+	ttype <- 'eta'
+	expect_error(fuhbar <- predict(fitnum,newdata=baddata,type=ttype,group=race))
 
 	expect_equal(as.numeric(coefficients(fitnum)),as.numeric(coefficients(fitlet)),tolerance=1e-7)
 	expect_equal(as.numeric(coefficients(fitnum)),as.numeric(coefficients(fitfac)),tolerance=1e-7)
@@ -655,6 +661,51 @@ test_that("hensm consistency",{#FOLDUP
 	expect_equal(fitm$gammas[1:3],gammas[1:3],tolerance=0.03)
 	expect_equal(as.numeric(fitm$beta),beta,tolerance=0.03)
 })#UNFOLD
+test_that("predictions with factors",{#FOLDUP
+	# travis only?
+	#skip_on_cran()
+	nfeat <- 2
+	set.seed(1234)
+	g <- ceiling(seq(0.1,30,by=0.1))
+	X <- matrix(rnorm(length(g) * nfeat),ncol=nfeat)
+	beta <- rnorm(nfeat)
+	eta <- X %*% beta
+	y <- rsm(eta,g=g)
+	
+	# create a variable V3, which is a factor variable.
+	# in the training data is has values 'a':'q'; we build a model.
+	# in the test data we try data with values 'a':'m' and only those levels.
+	# and also test data with values 'a':'m' but levels 'a':'z'.
+	# In each case we should be able to call predict on the model.
+	data <- cbind(data.frame(outcome=y,race=g),as.data.frame(X))
+	data_AQ <- data
+	data_AQ$V3 <- factor(sample(letters[1:17],nrow(data_AQ),replace=TRUE))
+	data_AM <- data
+	data_AM$V3 <- factor(sample(letters[1:13],nrow(data_AM),replace=TRUE))
+	data_AMZ <- data_AM
+	data_AMZ$V3 <- factor(as.character(data_AMZ$V3),levels=letters)
+
+	expect_error(fitmod <- harsm(outcome ~ V1 + V3,data_AQ,group=race),NA)
+	expect_error(henmod <- hensm(outcome ~ V1 + V3,data_AQ,group=race),NA)
+	for (ttype in c('eta','mu','erank')) {
+		expect_error(fuh <- predict(fitmod,newdata=data_AQ,type=ttype),NA)
+		expect_error(fuh <- predict(fitmod,newdata=data_AQ,type=ttype,group=race),NA)
+
+		expect_error(fuh <- predict(fitmod,newdata=data_AM,type=ttype),NA)
+		expect_error(fuh <- predict(fitmod,newdata=data_AM,type=ttype,group=race),NA)
+		expect_error(fuh <- predict(fitmod,newdata=data_AMZ,type=ttype),NA)
+		expect_error(fuh <- predict(fitmod,newdata=data_AMZ,type=ttype,group=race),NA)
+
+		expect_error(fuh <- predict(henmod,newdata=data_AQ,type=ttype),NA)
+		expect_error(fuh <- predict(henmod,newdata=data_AQ,type=ttype,group=race),NA)
+
+		expect_error(fuh <- predict(henmod,newdata=data_AM,type=ttype),NA)
+		expect_error(fuh <- predict(henmod,newdata=data_AM,type=ttype,group=race),NA)
+		expect_error(fuh <- predict(henmod,newdata=data_AMZ,type=ttype),NA)
+		expect_error(fuh <- predict(henmod,newdata=data_AMZ,type=ttype,group=race),NA)
+	}
+
+})#UNFOLD
 #UNFOLD
 context("weighting")#FOLDUP
 test_that("harsmfit zero weights",{#FOLDUP
